/*!
 * @file mapped_water.h
 *
*/
/* example input in .oxw file
<MappedWater>
  <waterpos>-500,0,-500</waterpos>
  <watersize>1000,1000</watersize>
  <wateruv>10</wateruv>
  <waterflow>0.003</waterflow>
  <lightpos>100,150,100</lightpos>
</MappedWater> */



#ifndef _MAPPED_WATER_H
#define _MAPPED_WATER_H

#include "world_entity.h"
#include "material.h"
#include "shader.h"
#include "effects/fog_effect.h"


class MappedWater : public WorldEntity
{
  public:
    MappedWater(const TiXmlElement* root = NULL);
    virtual ~MappedWater();

    void loadParams(const TiXmlElement* root);


    void activateReflection();
    void deactivateReflection();

    void activateRefraction();
    void deactivateRefraction();

    void draw() const;
    void tick(float dt);

    // functions to set parameters for the water, usually they're called through loadparam
    void setLightPosition(float x, float y, float z) { this->lightPos = Vector(x,y,z); };
    void setWaterPos(float x, float y, float z) { this->waterPos = Vector(x,y,z); };
    void setWaterSize(float x, float z) { this->xWidth = x; this->zWidth = z; };
    void setWaterUV(float uv) { this->g_WaterUV = uv; };
    void setWaterFlow(float flow) { this->g_WaterFlow = flow; };

  private:
    Vector              waterPos;                   //!< position of the water
    float               xWidth, zWidth;             //!< size of the water quad
    Vector              lightPos;                   //!< position of the light that is used to render the reflection

    float               move;                       //!< textures coords, speeds, positions for the shaded textures....
    float               move2;                      //!<
    float               g_WaterUV;                  //!<
    float               g_WaterFlow;                //!<
    float               refrUV;                     //!<
    float               normalUV;                   //!<
    float               kNormalMapScale;            //!<

    int                 textureSize;                //!< height and width of the texture
    Material            mat;
    Shader*             shader;
    Shader::Uniform*    cam_uni;                    //!< uniform that is used for the camera position

    //FogEffect*          fog;

};

#endif  /* _MAPPED_WATER_H */
